# frozen_string_literal: true

require 'uri'
require 'net/http'
require 'net/https'

module HintMe
  module API
    # Client for the DevHints API
    class Client
      # Base URL for the DevHints cheatsheets github repository
      RAW_REPO_URL = 'https://raw.githubusercontent.com/rstacruz/cheatsheets/master/'

      # searches and downloads the raw markdown representation of cheatsheet
      #
      # @param topic [String] topic to fetch cheatsheet for
      #
      def search_cheatsheet_for_topic(topic)
        download_cheatsheet(topic) if cheatsheet_exists?(topic)
      end

      private

      # checks if a cheatsheet for a certain topic exists
      #
      # @param topic [String]
      # @return [true, false]
      #
      def cheatsheet_exists?(topic)
        url = build_topic_url(topic)
        response = Net::HTTP.get_response(url)

        case response
        when Net::HTTPSuccess then true
        when Net::HTTPNotFound then false
        else
          handle_invalid_response(response)
        end
      end

      # downloads a cheatsheet for a given topic
      #
      # @param topic [String]
      # @return [String, nil] downloaded cheatsheet
      #
      def download_cheatsheet(topic)
        url = build_topic_url(topic)
        Net::HTTP.get(url)
                 .force_encoding('utf-8')
      end

      # builds the full repository url for a specific cheatsheet
      #
      # @param topic [String] topic for which the full cheatsheet url
      # should be build
      #
      def build_topic_url(topic)
        URI("#{RAW_REPO_URL}#{topic}.md")
      end

      def handle_invalid_response(response)
        raise StandartError, 'An network error occured.' \
          "Error response: #{response.value}"
      end
    end
  end
end
